package java.util;

import sun.util.locale.BaseLocale;
import sun.util.locale.LocaleObjectCache;
import java.util.jar.JarEntry;
import java.net.JarURLConnection;
import java.security.PrivilegedActionException;
import java.io.IOException;
import java.net.URLConnection;
import java.security.PrivilegedExceptionAction;
import java.lang.ref.SoftReference;
import java.lang.ref.WeakReference;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.io.InputStream;
import java.net.URL;
import java.util.concurrent.ConcurrentHashMap;
import java.lang.ref.Reference;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.util.spi.ResourceBundleControlProvider;
import java.lang.ref.ReferenceQueue;
import java.util.concurrent.ConcurrentMap;

public abstract class ResourceBundle
{
    private static final int INITIAL_CACHE_SIZE = 32;
    private static final ResourceBundle NONEXISTENT_BUNDLE;
    private static final ConcurrentMap<ResourceBundle.CacheKey, ResourceBundle.BundleReference> cacheList;
    private static final ReferenceQueue<Object> referenceQueue;
    protected ResourceBundle parent;
    private Locale locale;
    private String name;
    private volatile boolean expired;
    private volatile ResourceBundle.CacheKey cacheKey;
    private volatile Set<String> keySet;
    private static final List<ResourceBundleControlProvider> providers;
    
    public String getBaseBundleName() {
        return this.name;
    }
    
    public ResourceBundle() {
        this.parent = null;
        this.locale = null;
    }
    
    public final String getString(String s) {
        return (String)this.getObject(s);
    }
    
    public final String[] getStringArray(String s) {
        return (String[])(String[])this.getObject(s);
    }
    
    public final Object getObject(String str) {
        Object o = this.handleGetObject(str);
        if (o == null) {
            if (this.parent != null) {
                o = this.parent.getObject(str);
            }
            if (o == null) {
                throw new MissingResourceException("Can't find resource for bundle " + this.getClass().getName() + ", key " + str, this.getClass().getName(), str);
            }
        }
        return o;
    }
    
    public Locale getLocale() {
        return this.locale;
    }
    
    private static ClassLoader getLoader(Class<?> clazz) {
        ClassLoader access$000 = (clazz == null) ? null : clazz.getClassLoader();
        if (access$000 == null) {
            access$000 = ResourceBundle.RBClassLoader.INSTANCE;
            return access$000;
        }
        return access$000;
    }
    
    protected void setParent(ResourceBundle parent) {
        assert parent != ResourceBundle.NONEXISTENT_BUNDLE;
        this.parent = parent;
        return;
    }
    
    @CallerSensitive
    public static final ResourceBundle getBundle(String s) {
        return getBundleImpl(s, Locale.getDefault(), getLoader(Reflection.getCallerClass()), getDefaultControl(s));
    }
    
    @CallerSensitive
    public static final ResourceBundle getBundle(String s, ResourceBundle.Control control) {
        return getBundleImpl(s, Locale.getDefault(), getLoader(Reflection.getCallerClass()), control);
    }
    
    @CallerSensitive
    public static final ResourceBundle getBundle(String s, Locale locale) {
        return getBundleImpl(s, locale, getLoader(Reflection.getCallerClass()), getDefaultControl(s));
    }
    
    @CallerSensitive
    public static final ResourceBundle getBundle(String s, Locale locale, ResourceBundle.Control control) {
        return getBundleImpl(s, locale, getLoader(Reflection.getCallerClass()), control);
    }
    
    public static ResourceBundle getBundle(String s, Locale locale, ClassLoader classLoader) {
        if (classLoader == null) {
            throw new NullPointerException();
        }
        return getBundleImpl(s, locale, classLoader, getDefaultControl(s));
    }
    
    public static ResourceBundle getBundle(String s, Locale locale, ClassLoader classLoader, ResourceBundle.Control control) {
        if (classLoader == null || control == null) {
            throw new NullPointerException();
        }
        return getBundleImpl(s, locale, classLoader, control);
    }
    
    private static ResourceBundle.Control getDefaultControl(String s) {
        if (ResourceBundle.providers != null) {
            Iterator<ResourceBundleControlProvider> iterator = ResourceBundle.providers.iterator();
            while (iterator.hasNext()) {
                ResourceBundle.Control control = ((ResourceBundleControlProvider)iterator.next()).getControl(s);
                if (control != null) {
                    return control;
                }
                continue;
            }
        }
        return ResourceBundle.Control.INSTANCE;
    }
    
    private static ResourceBundle getBundleImpl(String s, Locale fallbackLocale, ClassLoader classLoader, ResourceBundle.Control control) {
        if (fallbackLocale == null || control == null) {
            throw new NullPointerException();
        }
        ResourceBundle.CacheKey cacheKey = new ResourceBundle.CacheKey(s, fallbackLocale, classLoader);
        ResourceBundle bundle = null;
        ResourceBundle.BundleReference bundleReference = (ResourceBundle.BundleReference)ResourceBundle.cacheList.get(cacheKey);
        if (bundleReference != null) {
            bundle = (ResourceBundle)bundleReference.get();
        }
        if (isValidBundle(bundle) && hasValidParentChain(bundle)) {
            return bundle;
        }
        int n = control == ResourceBundle.Control.INSTANCE || control instanceof ResourceBundle.SingleFormatControl;
        List<String> formats = control.getFormats(s);
        if (n == 0 && !checkList(formats)) {
            throw new IllegalArgumentException("Invalid Control: getFormats");
        }
        ResourceBundle resourceBundle = null;
        while (fallbackLocale != null) {
            List<Locale> candidateLocales = control.getCandidateLocales(s, fallbackLocale);
            if (n == 0 && !checkList(candidateLocales)) {
                throw new IllegalArgumentException("Invalid Control: getCandidateLocales");
            }
            bundle = findBundle(cacheKey, candidateLocales, formats, 0, control, resourceBundle);
            if (isValidBundle(bundle)) {
                int equals = Locale.ROOT.equals(bundle.locale);
                if (equals && !bundle.locale.equals(fallbackLocale)) {
                    if (candidateLocales.size() && bundle.locale.equals(candidateLocales.get(0))) {
                        break;
                    }
                    if (equals && resourceBundle == null) {
                        resourceBundle = bundle;
                    }
                }
                else {
                    break;
                }
            }
            fallbackLocale = control.getFallbackLocale(s, fallbackLocale);
        }
        if (bundle == null) {
            if (resourceBundle == null) {
                throwMissingResourceException(s, fallbackLocale, ResourceBundle.CacheKey.access$400(cacheKey));
            }
            bundle = resourceBundle;
        }
        keepAlive(classLoader);
        return bundle;
    }
    
    private static void keepAlive(ClassLoader classLoader) {
    }
    
    private static boolean checkList(List<?> list) {
        int n = list != null && !list.isEmpty();
        if (!(n == 0)) {
            for (int size = list.size(), n2 = 0; n && n2 < size; n = (list.get(n2) != null), ++n2) {}
        }
        return n;
    }
    
    private static ResourceBundle findBundle(ResourceBundle.CacheKey cacheKey, List<Locale> list, List<String> list2, int n, ResourceBundle.Control control, ResourceBundle resourceBundle) {
        Locale locale = (Locale)list.get(n);
        ResourceBundle bundle = null;
        if (n != list.size() - 1) {
            bundle = findBundle(cacheKey, list, list2, n + 1, control, resourceBundle);
        }
        else if (resourceBundle != null && Locale.ROOT.equals(locale)) {
            return resourceBundle;
        }
        Reference<Object> poll;
        while ((poll = ResourceBundle.referenceQueue.poll()) != null) {
            ResourceBundle.cacheList.remove(((ResourceBundle.CacheKeyReference)poll).getCacheKey());
        }
        boolean expired = false;
        cacheKey.setLocale(locale);
        ResourceBundle bundleInCache = findBundleInCache(cacheKey, control);
        if (isValidBundle(bundleInCache)) {
            expired = bundleInCache.expired;
            if (expired == 0) {
                if (bundleInCache.parent == bundle) {
                    return bundleInCache;
                }
                ResourceBundle.BundleReference bundleReference = (ResourceBundle.BundleReference)ResourceBundle.cacheList.get(cacheKey);
                if (bundleReference != null && bundleReference.get() == bundleInCache) {
                    ResourceBundle.cacheList.remove(cacheKey, bundleReference);
                }
            }
        }
        if (bundleInCache != ResourceBundle.NONEXISTENT_BUNDLE) {
            ResourceBundle.CacheKey cacheKey2 = (ResourceBundle.CacheKey)cacheKey.clone();
            try {
                ResourceBundle loadBundle = loadBundle(cacheKey, list2, control, (boolean)expired);
                if (loadBundle != null) {
                    if (loadBundle.parent == null) {
                        loadBundle.setParent(bundle);
                    }
                    loadBundle.locale = locale;
                    ResourceBundle putBundleInCache = putBundleInCache(cacheKey, loadBundle, control);
                }
                else {
                    putBundleInCache(cacheKey, ResourceBundle.NONEXISTENT_BUNDLE, control);
                }
            }
            finally {
                if ((ResourceBundle.CacheKey.access$400(cacheKey2) instanceof InterruptedException)) {
                    Thread.currentThread().interrupt();
                }
            }
        }
        return bundle;
    }
    
    private static ResourceBundle loadBundle(ResourceBundle.CacheKey cacheKey, List<String> list, ResourceBundle.Control control, boolean b) {
        Locale locale = cacheKey.getLocale();
        ResourceBundle bundle = null;
        int size = list.size();
        int i = 0;
        while (i < size) {
            String format = (String)list.get(i);
            try {
                bundle = control.newBundle(cacheKey.getName(), locale, format, cacheKey.getLoader(), b);
            }
            catch (LinkageError linkageError) {
                ResourceBundle.CacheKey.access$500(cacheKey, linkageError);
            }
            catch (Exception ex) {
                ResourceBundle.CacheKey.access$500(cacheKey, ex);
            }
            if (bundle != null) {
                cacheKey.setFormat(format);
                bundle.name = cacheKey.getName();
                bundle.locale = locale;
                bundle.expired = false;
                break;
            }
            ++i;
        }
        return bundle;
    }
    
    private static boolean isValidBundle(ResourceBundle resourceBundle) {
        return resourceBundle != null && resourceBundle != ResourceBundle.NONEXISTENT_BUNDLE;
    }
    
    private static boolean hasValidParentChain(ResourceBundle parent) {
        long currentTimeMillis = System.currentTimeMillis();
        while (parent != null) {
            if (parent.expired) {
                return false;
            }
            ResourceBundle.CacheKey cacheKey = parent.cacheKey;
            if (cacheKey != null) {
                long access$600 = cacheKey.expirationTime;
                if (access$600 >= 0L && access$600 <= currentTimeMillis) {
                    return false;
                }
            }
            parent = parent.parent;
        }
        return true;
    }
    
    private static void throwMissingResourceException(String s, Locale locale, Throwable t) {
        if ((t instanceof MissingResourceException)) {
            t = null;
        }
        throw new MissingResourceException("Can't find bundle for base name " + s + ", locale " + locale, s + "_" + locale, "", t);
    }
    
    private static ResourceBundle findBundleInCache(ResourceBundle.CacheKey cacheKey, ResourceBundle.Control control) {
        ResourceBundle.BundleReference bundleReference = (ResourceBundle.BundleReference)ResourceBundle.cacheList.get(cacheKey);
        if (bundleReference == null) {
            return null;
        }
        ResourceBundle resourceBundle = (ResourceBundle)bundleReference.get();
        if (resourceBundle == null) {
            return null;
        }
        ResourceBundle parent = resourceBundle.parent;
        assert parent != ResourceBundle.NONEXISTENT_BUNDLE;
        if (parent != null && parent.expired) {
            assert resourceBundle != ResourceBundle.NONEXISTENT_BUNDLE;
            resourceBundle.expired = true;
            resourceBundle.cacheKey = null;
            ResourceBundle.cacheList.remove(cacheKey, bundleReference);
            resourceBundle = null;
        }
        else {
            ResourceBundle.CacheKey cacheKey2 = bundleReference.getCacheKey();
            long access$600 = cacheKey2.expirationTime;
            if (!resourceBundle.expired && access$600 >= 0L && access$600 <= System.currentTimeMillis()) {
                if (resourceBundle != ResourceBundle.NONEXISTENT_BUNDLE) {
                    synchronized (resourceBundle) {
                        long access$601 = cacheKey2.expirationTime;
                        if (!resourceBundle.expired && access$601 >= 0L && access$601 <= System.currentTimeMillis()) {
                            try {
                                resourceBundle.expired = control.needsReload(cacheKey2.getName(), cacheKey2.getLocale(), cacheKey2.getFormat(), cacheKey2.getLoader(), resourceBundle, cacheKey2.loadTime);
                            }
                            catch (Exception ex) {
                                ResourceBundle.CacheKey.access$500(cacheKey, ex);
                            }
                            if (resourceBundle.expired) {
                                resourceBundle.cacheKey = null;
                                ResourceBundle.cacheList.remove(cacheKey, bundleReference);
                            }
                            else {
                                setExpirationTime(cacheKey2, control);
                            }
                        }
                    }
                }
                else {
                    ResourceBundle.cacheList.remove(cacheKey, bundleReference);
                    resourceBundle = null;
                }
            }
        }
        return resourceBundle;
    }
    
    private static ResourceBundle putBundleInCache(ResourceBundle.CacheKey cacheKey, ResourceBundle resourceBundle, ResourceBundle.Control control) {
        setExpirationTime(cacheKey, control);
        if (cacheKey.expirationTime != -1L) {
            ResourceBundle.CacheKey cacheKey2 = (ResourceBundle.CacheKey)cacheKey.clone();
            ResourceBundle.BundleReference bundleReference = new ResourceBundle.BundleReference(resourceBundle, ResourceBundle.referenceQueue, cacheKey2);
            resourceBundle.cacheKey = cacheKey2;
            ResourceBundle.BundleReference bundleReference2 = (ResourceBundle.BundleReference)ResourceBundle.cacheList.putIfAbsent(cacheKey2, bundleReference);
            if (bundleReference2 != null) {
                ResourceBundle resourceBundle2 = (ResourceBundle)bundleReference2.get();
                if (resourceBundle2 != null && !resourceBundle2.expired) {
                    resourceBundle.cacheKey = null;
                    resourceBundle = resourceBundle2;
                    bundleReference.clear();
                }
                else {
                    ResourceBundle.cacheList.put(cacheKey2, bundleReference);
                }
            }
        }
        return resourceBundle;
    }
    
    private static void setExpirationTime(ResourceBundle.CacheKey cacheKey, ResourceBundle.Control control) {
        long timeToLive = control.getTimeToLive(cacheKey.getName(), cacheKey.getLocale());
        if (timeToLive >= 0L) {
            long currentTimeMillis = System.currentTimeMillis();
            cacheKey.loadTime = currentTimeMillis;
            cacheKey.expirationTime = currentTimeMillis + timeToLive;
        }
        else if (timeToLive >= -2L) {
            cacheKey.expirationTime = timeToLive;
        }
        else {
            throw new IllegalArgumentException("Invalid Control: TTL=" + timeToLive);
        }
    }
    
    @CallerSensitive
    public static final void clearCache() {
        clearCache(getLoader(Reflection.getCallerClass()));
    }
    
    public static final void clearCache(ClassLoader classLoader) {
        if (classLoader == null) {
            throw new NullPointerException();
        }
        Set<Object> keySet = ResourceBundle.cacheList.keySet();
        for (ResourceBundle.CacheKey cacheKey : keySet) {
            if (cacheKey.getLoader() == classLoader) {
                keySet.remove(cacheKey);
            }
            else {
                continue;
            }
        }
        return;
    }
    
    protected abstract Object handleGetObject(String p0);
    
    public abstract Enumeration<String> getKeys();
    
    public boolean containsKey(String s) {
        if (s == null) {
            throw new NullPointerException();
        }
        while (this != null) {
            if (this.handleKeySet().contains(s)) {
                return true;
            }
            this = this.parent;
        }
        return false;
    }
    
    public Set<String> keySet() {
        HashSet<Object> set = new HashSet();
        while (this != null) {
            set.addAll(this.handleKeySet());
            this = this.parent;
        }
        return set;
    }
    
    protected Set<String> handleKeySet() {
        if (this.keySet == null) {
            synchronized (this) {
                if (this.keySet == null) {
                    HashSet<String> keySet = new HashSet();
                    Enumeration<String> keys = this.getKeys();
                    while (keys.hasMoreElements()) {
                        String s = (String)keys.nextElement();
                        if (this.handleGetObject(s) != null) {
                            keySet.add(s);
                        }
                        else {
                            continue;
                        }
                    }
                    this.keySet = keySet;
                }
            }
        }
        return this.keySet;
    }
    
    static {
        NONEXISTENT_BUNDLE = new ResourceBundle() {
            @Override
            public Enumeration<String> getKeys() {
                return null;
            }
            
            @Override
            protected Object handleGetObject(String s) {
                return null;
            }
            
            @Override
            public String toString() {
                return "NONEXISTENT_BUNDLE";
            }
        };
        cacheList = new ConcurrentHashMap(32);
        referenceQueue = new ReferenceQueue();
        Object providers2 = null;
        for (ResourceBundleControlProvider resourceBundleControlProvider : ServiceLoader.loadInstalled(ResourceBundleControlProvider.class)) {
            if (providers2 == null) {
                providers2 = new ArrayList();
            }
            providers2.add(resourceBundleControlProvider);
        }
        providers = providers2;
    }
    
    private static class RBClassLoader extends ClassLoader
    {
        private static final ResourceBundle.RBClassLoader INSTANCE;
        private static final ClassLoader loader;
        
        @Override
        public Class<?> loadClass(String s) throws ClassNotFoundException {
            if (ResourceBundle.RBClassLoader.loader != null) {
                return ResourceBundle.RBClassLoader.loader.loadClass(s);
            }
            return Class.forName(s);
        }
        
        @Override
        public URL getResource(String s) {
            if (ResourceBundle.RBClassLoader.loader != null) {
                return ResourceBundle.RBClassLoader.loader.getResource(s);
            }
            return ClassLoader.getSystemResource(s);
        }
        
        @Override
        public InputStream getResourceAsStream(String s) {
            if (ResourceBundle.RBClassLoader.loader != null) {
                return ResourceBundle.RBClassLoader.loader.getResourceAsStream(s);
            }
            return ClassLoader.getSystemResourceAsStream(s);
        }
        
        static {
            INSTANCE = (ResourceBundle.RBClassLoader)AccessController.doPrivileged(new PrivilegedAction<ResourceBundle.RBClassLoader>() {
                @Override
                public ResourceBundle.RBClassLoader run() {
                    return new ResourceBundle.RBClassLoader();
                }
            });
            ClassLoader systemClassLoader;
            ClassLoader parent;
            for (systemClassLoader = ClassLoader.getSystemClassLoader(); (parent = systemClassLoader.getParent()) != null; systemClassLoader = parent) {}
            loader = systemClassLoader;
        }
    }
    
    private static class CacheKey implements Cloneable
    {
        private String name;
        private Locale locale;
        private ResourceBundle.LoaderReference loaderRef;
        private String format;
        private volatile long loadTime;
        private volatile long expirationTime;
        private Throwable cause;
        private int hashCodeCache;
        
        CacheKey(String name, Locale locale, ClassLoader classLoader) {
            this.name = name;
            this.locale = locale;
            if (classLoader == null) {
                this.loaderRef = null;
            }
            else {
                this.loaderRef = new ResourceBundle.LoaderReference(classLoader, ResourceBundle.referenceQueue, this);
            }
            this.calculateHashCode();
        }
        
        String getName() {
            return this.name;
        }
        
        ResourceBundle.CacheKey setName(String s) {
            if (!this.name.equals(s)) {
                this.name = s;
                this.calculateHashCode();
                return this;
            }
            return this;
        }
        
        Locale getLocale() {
            return this.locale;
        }
        
        ResourceBundle.CacheKey setLocale(Locale locale) {
            if (!this.locale.equals(locale)) {
                this.locale = locale;
                this.calculateHashCode();
                return this;
            }
            return this;
        }
        
        ClassLoader getLoader() {
            return (this.loaderRef != null) ? ((ClassLoader)this.loaderRef.get()) : null;
        }
        
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            try {
                ResourceBundle.CacheKey cacheKey = (ResourceBundle.CacheKey)o;
                if (this.hashCodeCache != cacheKey.hashCodeCache) {
                    return false;
                }
                if (!this.name.equals(cacheKey.name)) {
                    return false;
                }
                if (!this.locale.equals(cacheKey.locale)) {
                    return false;
                }
                if (this.loaderRef == null) {
                    return cacheKey.loaderRef == null;
                }
                ClassLoader classLoader = (ClassLoader)this.loaderRef.get();
                return cacheKey.loaderRef != null && classLoader != null && classLoader == cacheKey.loaderRef.get();
            }
            catch (NullPointerException | ClassCastException ex) {
                return false;
            }
        }
        
        @Override
        public int hashCode() {
            return this.hashCodeCache;
        }
        
        private void calculateHashCode() {
            this.hashCodeCache = this.name.hashCode() << 3;
            this.hashCodeCache ^= this.locale.hashCode();
            ClassLoader loader = this.getLoader();
            if (loader != null) {
                this.hashCodeCache ^= loader.hashCode();
            }
        }
        
        public Object clone() {
            try {
                ResourceBundle.CacheKey cacheKey = (ResourceBundle.CacheKey)super.clone();
                if (this.loaderRef != null) {
                    cacheKey.loaderRef = new ResourceBundle.LoaderReference((ClassLoader)this.loaderRef.get(), ResourceBundle.referenceQueue, cacheKey);
                }
                cacheKey.cause = null;
                return cacheKey;
            }
            catch (CloneNotSupportedException cause) {
                throw new InternalError(cause);
            }
        }
        
        String getFormat() {
            return this.format;
        }
        
        void setFormat(String format) {
            this.format = format;
        }
        
        private void setCause(Throwable t) {
            if (this.cause == null) {
                this.cause = t;
            }
            else if ((this.cause instanceof ClassNotFoundException)) {
                this.cause = t;
            }
        }
        
        private Throwable getCause() {
            return this.cause;
        }
        
        @Override
        public String toString() {
            String str = this.locale.toString();
            if (str.length() == 0) {
                if (this.locale.getVariant().length() != 0) {
                    str = "__" + this.locale.getVariant();
                }
                else {
                    str = "\"\"";
                }
            }
            return "CacheKey[" + this.name + ", lc=" + str + ", ldr=" + this.getLoader() + "(format=" + this.format + ")]";
        }
        
        static Throwable access$400(ResourceBundle.CacheKey cacheKey) {
            return cacheKey.getCause();
        }
        
        static void access$500(ResourceBundle.CacheKey cacheKey, Throwable cause) {
            cacheKey.setCause(cause);
        }
    }
    
    private static class LoaderReference extends WeakReference<ClassLoader> implements ResourceBundle.CacheKeyReference
    {
        private ResourceBundle.CacheKey cacheKey;
        
        LoaderReference(ClassLoader referent, ReferenceQueue<Object> q, ResourceBundle.CacheKey cacheKey) {
            super(referent, q);
            this.cacheKey = cacheKey;
        }
        
        @Override
        public ResourceBundle.CacheKey getCacheKey() {
            return this.cacheKey;
        }
    }
    
    private static class BundleReference extends SoftReference<ResourceBundle> implements ResourceBundle.CacheKeyReference
    {
        private ResourceBundle.CacheKey cacheKey;
        
        BundleReference(ResourceBundle referent, ReferenceQueue<Object> q, ResourceBundle.CacheKey cacheKey) {
            super(referent, q);
            this.cacheKey = cacheKey;
        }
        
        @Override
        public ResourceBundle.CacheKey getCacheKey() {
            return this.cacheKey;
        }
    }
    
    public static class Control
    {
        public static final List<String> FORMAT_DEFAULT;
        public static final List<String> FORMAT_CLASS;
        public static final List<String> FORMAT_PROPERTIES;
        public static final long TTL_DONT_CACHE = -1L;
        public static final long TTL_NO_EXPIRATION_CONTROL = -2L;
        private static final ResourceBundle.Control INSTANCE;
        private static final ResourceBundle.Control.CandidateListCache CANDIDATES_CACHE;
        
        protected Control() {
        }
        
        public static final ResourceBundle.Control getControl(List<String> list) {
            if (list.equals(ResourceBundle.Control.FORMAT_PROPERTIES)) {
                return ResourceBundle.SingleFormatControl.PROPERTIES_ONLY;
            }
            if (list.equals(ResourceBundle.Control.FORMAT_CLASS)) {
                return ResourceBundle.SingleFormatControl.CLASS_ONLY;
            }
            if (list.equals(ResourceBundle.Control.FORMAT_DEFAULT)) {
                return ResourceBundle.Control.INSTANCE;
            }
            throw new IllegalArgumentException();
        }
        
        public static final ResourceBundle.Control getNoFallbackControl(List<String> list) {
            if (list.equals(ResourceBundle.Control.FORMAT_DEFAULT)) {
                return ResourceBundle.NoFallbackControl.NO_FALLBACK;
            }
            if (list.equals(ResourceBundle.Control.FORMAT_PROPERTIES)) {
                return ResourceBundle.NoFallbackControl.PROPERTIES_ONLY_NO_FALLBACK;
            }
            if (list.equals(ResourceBundle.Control.FORMAT_CLASS)) {
                return ResourceBundle.NoFallbackControl.CLASS_ONLY_NO_FALLBACK;
            }
            throw new IllegalArgumentException();
        }
        
        public List<String> getFormats(String s) {
            if (s == null) {
                throw new NullPointerException();
            }
            return ResourceBundle.Control.FORMAT_DEFAULT;
        }
        
        public List<Locale> getCandidateLocales(String s, Locale locale) {
            if (s == null) {
                throw new NullPointerException();
            }
            return new ArrayList((Collection)ResourceBundle.Control.CANDIDATES_CACHE.get(locale.getBaseLocale()));
        }
        
        public Locale getFallbackLocale(String s, Locale locale) {
            if (s == null) {
                throw new NullPointerException();
            }
            Locale default1 = Locale.getDefault();
            return locale.equals(default1) ? null : default1;
        }
        
        public ResourceBundle newBundle(String s, Locale locale, String str, ClassLoader classLoader, boolean b) throws IllegalAccessException, InstantiationException, IOException {
            String bundleName = this.toBundleName(s, locale);
            ResourceBundle resourceBundle = null;
            if (str.equals("java.class")) {
                try {
                    Class<Object> loadClass = classLoader.loadClass(bundleName);
                    if (ResourceBundle.class.isAssignableFrom(loadClass)) {
                        resourceBundle = (ResourceBundle)loadClass.newInstance();
                    }
                    else {
                        throw new ClassCastException(loadClass.getName() + " cannot be cast to ResourceBundle");
                    }
                }
                catch (ClassNotFoundException ex2) {}
            }
            else if (str.equals("java.properties")) {
                String resourceName0 = this.toResourceName0(bundleName, "properties");
                if (resourceName0 == null) {
                    return resourceBundle;
                }
                InputStream inputStream;
                try {
                    inputStream = (InputStream)AccessController.doPrivileged(new PrivilegedExceptionAction<InputStream>() {
                        @Override
                        public InputStream run() throws IOException {
                            InputStream inputStream = null;
                            if (b) {
                                URL resource = classLoader.getResource(resourceName0);
                                if (resource != null) {
                                    URLConnection openConnection = resource.openConnection();
                                    if (openConnection != null) {
                                        openConnection.setUseCaches(false);
                                        inputStream = openConnection.getInputStream();
                                    }
                                }
                                return inputStream;
                            }
                            else {
                                inputStream = classLoader.getResourceAsStream(resourceName0);
                                return inputStream;
                            }
                        }
                    });
                }
                catch (PrivilegedActionException ex) {
                    throw (IOException)ex.getException();
                }
                if (inputStream != null) {
                    try {
                        resourceBundle = new PropertyResourceBundle(inputStream);
                    }
                    finally {
                        inputStream.close();
                    }
                }
            }
            else {
                throw new IllegalArgumentException("unknown format: " + str);
            }
            return resourceBundle;
        }
        
        public long getTimeToLive(String s, Locale locale) {
            if (s == null || locale == null) {
                throw new NullPointerException();
            }
            return -2L;
        }
        
        public boolean needsReload(String s, Locale locale, String substring, ClassLoader classLoader, ResourceBundle resourceBundle, long n) {
            if (resourceBundle == null) {
                throw new NullPointerException();
            }
            if (substring.equals("java.class") || substring.equals("java.properties")) {
                substring = substring.substring(5);
            }
            boolean n2 = false;
            try {
                String resourceName0 = this.toResourceName0(this.toBundleName(s, locale), substring);
                if (resourceName0 == null) {
                    return n2;
                }
                URL resource = classLoader.getResource(resourceName0);
                if (resource != null) {
                    long n3 = 0L;
                    URLConnection openConnection = resource.openConnection();
                    if (openConnection != null) {
                        openConnection.setUseCaches(false);
                        if ((openConnection instanceof JarURLConnection)) {
                            JarEntry jarEntry = ((JarURLConnection)openConnection).getJarEntry();
                            if (jarEntry != null) {
                                n3 = jarEntry.getTime();
                                if (n3 == -1L) {
                                    n3 = 0L;
                                }
                            }
                        }
                        else {
                            n3 = openConnection.getLastModified();
                        }
                    }
                    n2 = (n3 >= n);
                }
            }
            catch (NullPointerException ex) {
                throw ex;
            }
            catch (Exception ex2) {}
            return n2;
        }
        
        public String toBundleName(String str, Locale locale) {
            if (locale == Locale.ROOT) {
                return str;
            }
            String language = locale.getLanguage();
            String script = locale.getScript();
            String country = locale.getCountry();
            String variant = locale.getVariant();
            if (language == "" && country == "" && variant == "") {
                return str;
            }
            StringBuilder sb = new StringBuilder(str);
            sb.append('_');
            if (script != "") {
                if (variant != "") {
                    sb.append(language).append('_').append(script).append('_').append(country).append('_').append(variant);
                }
                else if (country != "") {
                    sb.append(language).append('_').append(script).append('_').append(country);
                }
                else {
                    sb.append(language).append('_').append(script);
                }
            }
            else if (variant != "") {
                sb.append(language).append('_').append(country).append('_').append(variant);
            }
            else if (country != "") {
                sb.append(language).append('_').append(country);
            }
            else {
                sb.append(language);
            }
            return sb.toString();
        }
        
        public final String toResourceName(String s, String str) {
            StringBuilder sb = new StringBuilder(s.length() + 1 + str.length());
            sb.append(s.replace('.', '/')).append('.').append(str);
            return sb.toString();
        }
        
        private String toResourceName0(String s, String s2) {
            if (s.contains("://")) {
                return null;
            }
            return this.toResourceName(s, s2);
        }
        
        static {
            FORMAT_DEFAULT = Collections.unmodifiableList(Arrays.asList(new String[] { "java.class", "java.properties" }));
            FORMAT_CLASS = Collections.unmodifiableList(Arrays.asList(new String[] { "java.class" }));
            FORMAT_PROPERTIES = Collections.unmodifiableList(Arrays.asList(new String[] { "java.properties" }));
            INSTANCE = new ResourceBundle.Control();
            CANDIDATES_CACHE = new ResourceBundle.Control.CandidateListCache();
        }
        
        private static class CandidateListCache extends LocaleObjectCache<BaseLocale, List<Locale>>
        {
            @Override
            protected List<Locale> createObject(BaseLocale baseLocale) {
                String language = baseLocale.getLanguage();
                String script = baseLocale.getScript();
                String region = baseLocale.getRegion();
                String variant = baseLocale.getVariant();
                boolean n = false;
                boolean n2 = false;
                if (language.equals("no")) {
                    if (region.equals("NO") && variant.equals("NY")) {
                        variant = "";
                        n2 = true;
                    }
                    else {
                        n = true;
                    }
                }
                if (language.equals("nb") || n) {
                    List<Locale> defaultList2 = getDefaultList("nb", script, region, variant);
                    LinkedList o = new LinkedList();
                    for (Locale locale : defaultList2) {
                        o.add(locale);
                        if (locale.getLanguage().length() == 0) {
                            break;
                        }
                        o.add(Locale.getInstance("no", locale.getScript(), locale.getCountry(), locale.getVariant(), null));
                    }
                    return o;
                }
                if (language.equals("nn") || n2) {
                    List defaultList = getDefaultList("nn", script, region, variant);
                    int n5 = defaultList.size() - 1;
                    defaultList.add(n5++, Locale.getInstance("no", "NO", "NY"));
                    defaultList.add(n5++, Locale.getInstance("no", "NO", ""));
                    defaultList.add(n5++, Locale.getInstance("no", "", ""));
                    return defaultList;
                }
                if (language.equals("zh")) {
                    if (script.length() == 0 && region.length() > 0) {
                        int n4 = -1;
                        switch (region.hashCode()) {
                            case 2691: {
                                if (region.equals("TW")) {
                                    n4 = 0;
                                    break;
                                }
                                break;
                            }
                            case 2307: {
                                if (region.equals("HK")) {
                                    n4 = 1;
                                    break;
                                }
                                break;
                            }
                            case 2466: {
                                if (region.equals("MO")) {
                                    n4 = 2;
                                    break;
                                }
                                break;
                            }
                            case 2155: {
                                if (region.equals("CN")) {
                                    n4 = 3;
                                    break;
                                }
                                break;
                            }
                            case 2644: {
                                if (region.equals("SG")) {
                                    n4 = 4;
                                    break;
                                }
                                break;
                            }
                        }
                        switch (n4) {
                            case 0:
                            case 1:
                            case 2: {
                                script = "Hant";
                                break;
                            }
                            case 3:
                            case 4: {
                                script = "Hans";
                                break;
                            }
                        }
                    }
                    else if (script.length() > 0 && region.length() == 0) {
                        int n3 = -1;
                        switch (script.hashCode()) {
                            case 2241694: {
                                if (script.equals("Hans")) {
                                    n3 = 0;
                                    break;
                                }
                                break;
                            }
                            case 2241695: {
                                if (script.equals("Hant")) {
                                    n3 = 1;
                                    break;
                                }
                                break;
                            }
                        }
                        switch (n3) {
                            case 0: {
                                region = "CN";
                                break;
                            }
                            case 1: {
                                region = "TW";
                                break;
                            }
                        }
                    }
                }
                return getDefaultList(language, script, region, variant);
            }
            
            private static List<Locale> getDefaultList(String s, String s2, String s3, String s4) {
                LinkedList<String> list = null;
                if (s4.length() > 0) {
                    list = new LinkedList();
                    for (int i = s4.length(); i != -1; i = s4.lastIndexOf(95, --i)) {
                        list.add(s4.substring(0, i));
                    }
                }
                LinkedList o = new LinkedList();
                if (list != null) {
                    Iterator<Object> iterator = list.iterator();
                    while (iterator.hasNext()) {
                        o.add(Locale.getInstance(s, s2, s3, (String)iterator.next(), null));
                    }
                }
                if (s3.length() > 0) {
                    o.add(Locale.getInstance(s, s2, s3, "", null));
                }
                if (s2.length() > 0) {
                    o.add(Locale.getInstance(s, s2, "", "", null));
                    if (list != null) {
                        Iterator<Object> iterator2 = list.iterator();
                        while (iterator2.hasNext()) {
                            o.add(Locale.getInstance(s, "", s3, (String)iterator2.next(), null));
                        }
                    }
                    if (s3.length() > 0) {
                        o.add(Locale.getInstance(s, "", s3, "", null));
                    }
                }
                if (s.length() > 0) {
                    o.add(Locale.getInstance(s, "", "", "", null));
                }
                o.add(Locale.ROOT);
                return o;
            }
        }
    }
    
    private static class SingleFormatControl extends ResourceBundle.Control
    {
        private static final ResourceBundle.Control PROPERTIES_ONLY;
        private static final ResourceBundle.Control CLASS_ONLY;
        private final List<String> formats;
        
        protected SingleFormatControl(List<String> formats) {
            this.formats = formats;
        }
        
        @Override
        public List<String> getFormats(String s) {
            if (s == null) {
                throw new NullPointerException();
            }
            return this.formats;
        }
        
        static {
            PROPERTIES_ONLY = new ResourceBundle.SingleFormatControl(ResourceBundle.SingleFormatControl.FORMAT_PROPERTIES);
            CLASS_ONLY = new ResourceBundle.SingleFormatControl(ResourceBundle.SingleFormatControl.FORMAT_CLASS);
        }
    }
    
    private static final class NoFallbackControl extends ResourceBundle.SingleFormatControl
    {
        private static final ResourceBundle.Control NO_FALLBACK;
        private static final ResourceBundle.Control PROPERTIES_ONLY_NO_FALLBACK;
        private static final ResourceBundle.Control CLASS_ONLY_NO_FALLBACK;
        
        protected NoFallbackControl(List<String> list) {
            super(list);
        }
        
        @Override
        public Locale getFallbackLocale(String s, Locale locale) {
            if (s == null || locale == null) {
                throw new NullPointerException();
            }
            return null;
        }
        
        static {
            NO_FALLBACK = new ResourceBundle.NoFallbackControl(ResourceBundle.NoFallbackControl.FORMAT_DEFAULT);
            PROPERTIES_ONLY_NO_FALLBACK = new ResourceBundle.NoFallbackControl(ResourceBundle.NoFallbackControl.FORMAT_PROPERTIES);
            CLASS_ONLY_NO_FALLBACK = new ResourceBundle.NoFallbackControl(ResourceBundle.NoFallbackControl.FORMAT_CLASS);
        }
    }
    
    private interface CacheKeyReference
    {
        public abstract ResourceBundle.CacheKey getCacheKey();
    }
}
